package serviceController;

import java.io.BufferedReader;
import java.io.File;
//import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
//import java.util.Date;

import toolBox.ConfigManager;
//import toolBox.ErrorLog;
import toolBox.LogAnalyser;
//import twitterClient.TwitterClient;
//import twitterServer.TwitterServer;

public class ServiceController {
	
	public static int MaxNumOfProcesses; // The largest permitted number of TwitterClient processes 
	public static int MaxNumOfThreads; // The largest permitted number of threads contained within one TwitterClient process
	
	public static String TwitterClientWorkingDir; // The working directory of TwitterClient
	public static String TwitterClientLogDir; // The log directory of TwitterClient
	public static String TwitterClientPath; // The path of TwitterClient
	
	public static String TwitterServerWorkingDir; // The working directory of TwitterServer
	public static String TwitterServerLogDir; // The log directory of TwitterServer
	public static String TwitterServerPath; // The path of TwitterServer
	public static int NumOfServers; // The number of TwitterServer(s) we can start
	
	private static int NumOfUserIDs;
	private static int NumOfProcesses;
	
	public static void main(String[] args){
		
		if(ConfigManager.LoadServiceControllerConfig() != 0){
			System.out.println("Fatal error: Fail to initialize ServiceController.");
			System.exit(-1);
		}
		
		if(args.length == 1 && args[0].equalsIgnoreCase("data")){
				System.out.println("Preparing data...");
				DataHandler.HandleData(0x03); // Prepare data set for TwitterClient and TwitterServer
				System.out.println("Data are ready.");
		}else if(args.length == 1 && args[0].equalsIgnoreCase("clientdata")){
				System.out.println("Preparing data for TwitterClient(s)...");
				DataHandler.HandleData(0x01); // Only prepare data set for TwitterClient
				System.out.println("You can start TwitterClient(s) now.");
		}else if(args.length == 1 && args[0].equalsIgnoreCase("serverdata")){
				System.out.println("Preparing data for TwitterServer(s)...");
				DataHandler.HandleData(0x02); // Only prepare data set for TwitterServer
				System.out.println("You can start TwitterServer(s) now.");
		}else if(args.length == 1 && args[0].equalsIgnoreCase("exception")){
				LogAnalyser.FindInLogFiles("Exception",TwitterServerLogDir);
				LogAnalyser.FindInLogFiles("Exception",TwitterClientLogDir);
		}
		else if(args.length == 1 && args[0].equalsIgnoreCase("client")){
				ClientController(); // Start several clients
		}else if(args.length >= 1 && args[0].equalsIgnoreCase("server")){
				ServerController(args); // Start several servers
		}else if(args.length >= 1){ // Find the designated string in all long files
				String ObjectString = new String(" ");
				for(int i = 0; i < args.length; i++)
					ObjectString = ObjectString + args[i] + " ";
				LogAnalyser.FindInLogFiles(ObjectString.trim(),TwitterServerLogDir);
				LogAnalyser.FindInLogFiles(ObjectString.trim(),TwitterClientLogDir);
		}else{
			System.out.println("Input error.");
			System.exit(-1);
		}
	}
	
	private static void ClientController(){
	
		ArrayList<Process> ProcessArray = null;
		
		try {
		    File Folder = new File(TwitterClientWorkingDir);
		    File[] ListOfFolders = Folder.listFiles();
		    String UserIDLists = new String(TwitterClientLogDir + "TwitterClient0.log"); // The fist substring of the UserIDLists is not a real UserID but the log file name.
		    Runtime r = Runtime.getRuntime();
		    ProcessArray = new ArrayList<Process>();
		    Process pProcess = null;
		    
		    class ClientProcess extends Thread {
				private Process CltProcess;
				private String InptStream;
				
				ClientProcess(Process SvrProcess, String StreamString){
					this.CltProcess = SvrProcess;
					this.InptStream = StreamString;
				}
					
				public void run(){
					String OneLine = null;
					BufferedReader br = null;
					
					try {
						if(InptStream.equals("ErrorStream"))
							br = new BufferedReader(new InputStreamReader(CltProcess.getErrorStream()));
						else if(InptStream.equals("InputStream"))
							br = new BufferedReader(new InputStreamReader(CltProcess.getInputStream()));
						
						while((OneLine = br.readLine()) != null)
							printToConsole(OneLine);
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				
				private synchronized void printToConsole(String Line){
					System.out.println(Line);
				}
			}
		    
		    for (;NumOfUserIDs < ListOfFolders.length && NumOfUserIDs < MaxNumOfThreads * MaxNumOfProcesses; NumOfUserIDs++) {
		    	if (ListOfFolders[NumOfUserIDs].isDirectory()){

		    		UserIDLists += " ";
		    		UserIDLists += ListOfFolders[NumOfUserIDs].getName();
		    		
		    		if((NumOfUserIDs + 1) % MaxNumOfThreads == 0){
		    			try{
		    				pProcess = r.exec("java -jar " + TwitterClientPath + " " + UserIDLists.trim()); // This MUST be changed if the jar file is put in some other place.
		    				NumOfProcesses++;
		    				if(pProcess != null){
		    					ProcessArray.add(pProcess);
		    					new ClientProcess(pProcess, "ErrorStream").start();
		    					new ClientProcess(pProcess, "InputStream").start();
		    				}
		    				pProcess = null;
		    				UserIDLists = TwitterClientLogDir + "TwitterClient" + Long.toString(NumOfProcesses) + ".log";
		    			}catch(Exception e){
		    				e.printStackTrace();
		    				pProcess = null;
		    				continue;
		    			}
		    		}
		    	}
		    }
		    
		    if(UserIDLists.contains(" ")){ // There are still some IDs in this list
		    	try{
		    		Process p = r.exec("java -jar " + TwitterClientPath + " " + UserIDLists.trim()); // The same as the above
		    		NumOfProcesses++;
		    		if(p != null){
		    			ProcessArray.add(p);
		    			new ClientProcess(p, "ErrorStream").start();
		    			new ClientProcess(p, "InputStream").start();
		    		}	
		    	}catch(Exception e){
		    		e.printStackTrace();
		    	}
		    }
		    
		}catch (Exception e) {
		 	e.printStackTrace();
		}finally{
			System.out.println("Find " + Long.toString(NumOfUserIDs)+ " user ID(s).");
			System.out.println("Create " + Long.toString(NumOfProcesses)+ " TwitterClient(s).");
			
			try{
				class TempThread extends Thread {
					private ArrayList<Process> PArray;
					private Thread Thd;
					
					TempThread(ArrayList<Process> PArray, Thread Thd){
						this.PArray = PArray;
						this.Thd = Thd;
					}
					
					public void run(){
						System.out.println("Wait for all TwitterClient(s) to complete.");
						try {
							Thd.interrupt();
							for(int i = 0; i < NumOfProcesses; i++)
								if(PArray.get(i) != null)
										PArray.get(i).waitFor();
						} catch (InterruptedException e) {
							for(int i = 0; i < NumOfProcesses; i++)
								if(PArray.get(i) != null)
										PArray.get(i).destroy();
							System.out.println("All TwitterClient(s) have stopped.");
							return;
						} 
						System.out.println("\nAll TwitterClient(s) have stopped.");
						System.exit(0);
					}
				}
				
				Thread Waiting = new TempThread(ProcessArray, Thread.currentThread());
				Waiting.start();
			
				try{
					Thread.sleep(10000); // Sleep 10 seconds here for the "TempThread" to fully start. 
				} catch(InterruptedException e) {
					// Do nothing here...
				}
				
				System.out.println("Type \"exit\" to stop all TwitterClient(s):");
				 
				BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
				while(true){
					String s = br.readLine();
				    if(s != null && s.equalsIgnoreCase("exit")){
				    	Waiting.interrupt();
				    	System.exit(0);
				    }
				    else if(s != null){
				    	System.out.println("Command not supported.");
				    }
				    else
				    	System.exit(0);
				}
			}catch(Exception e){
				e.printStackTrace();
				System.exit(0);
			}
		}
	}
	
	private static void ServerController(String args[]){
		
		int SvrCnt = Integer.MAX_VALUE; 
		
		if(args.length == 2){
			SvrCnt = Integer.valueOf(args[1]);
		}else if(args.length != 1){
			System.out.println("Input parameter error.");
			return;
		}
		
		try {
			SvrCnt = SvrCnt < NumOfServers ? SvrCnt : NumOfServers;
			
			int SuccessfulServerCnt = 0;
			Process[] ProcessArray = new Process[SvrCnt];
			
			class ServerProcess extends Thread {
				private Process SvrProcess;
				private String InptStream;
				
				ServerProcess(Process SvrProcess, String StreamString){
					this.SvrProcess = SvrProcess;
					this.InptStream = StreamString;
				}
				
				private synchronized void printToConsole(String Line){
					System.out.println(Line);
				}
				
				public void run(){
					String OneLine = null;
					BufferedReader br = null;
					
					try {
						if(InptStream.equals("ErrorStream"))
							br = new BufferedReader(new InputStreamReader(SvrProcess.getErrorStream()));
						else if(InptStream.equals("InputStream"))
							br = new BufferedReader(new InputStreamReader(SvrProcess.getInputStream()));
						
						while((OneLine = br.readLine()) != null){
							printToConsole(OneLine);
						}
					} catch (IOException e) {
						e.printStackTrace();
					}
				}	
			}
			
			class CommandInputer extends Thread {
				private Process[] SvrProcess;
				private PrintWriter[] pw;
				private int SvrCnt;
				
				CommandInputer(Process[] SvrProcess, PrintWriter[] pw, int ServerCount){
					this.SvrProcess = SvrProcess;
					this.pw = pw;
					this.SvrCnt = ServerCount; 
				}
				
				public void run(){
					try {
						while(true){
							sleep(5000); // Sleep five seconds
						
							for(int i = 0; i < SvrCnt; i++){
				    			if(SvrProcess[i] != null){
				    				try{
				    					pw[i].println("stat");
				    					pw[i].flush();
				    				}catch(Exception e){
				    					System.out.println("Cannot write command \"stat\" to TwitterServer.");
				    					continue;
				    				}
				    			}
							}
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}	
			}
			
			PrintWriter[] pw = new PrintWriter[SvrCnt];
			for(int i = 0; i < SvrCnt; i++){
				ProcessArray[i] = Runtime.getRuntime().exec("java -jar " + TwitterServerPath);
				if(ProcessArray[i] != null){
					SuccessfulServerCnt++;
					pw[i] = new PrintWriter(new OutputStreamWriter(ProcessArray[i].getOutputStream()));
				}
			}
			
			System.out.println("Create " + Long.toString(SuccessfulServerCnt)+ " TwitterServer(s).");
			System.out.println("Type \"exit\" to stop all TwitterServer(s):");
			
			for(int i = 0; i < SvrCnt; i++){
				if(ProcessArray[i] != null){
					new ServerProcess(ProcessArray[i],"ErrorStream").start();
					new ServerProcess(ProcessArray[i],"InputStream").start();
				}
			}
			
			new CommandInputer(ProcessArray, pw, SvrCnt).start();
			
			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
			while(true){
				String s = br.readLine();
			    if(s != null && s.equalsIgnoreCase("exit")){
			    	for(int i = 0; i < SvrCnt; i++)
			    		if(ProcessArray[i] != null)
			    			ProcessArray[i].destroy();
			    	System.out.println("All TwitterServer(s) have stopped.");
			    	System.exit(0);
			    }
			   
			    	/*
			    	BufferedReader[] brLogUserNumbers = new BufferedReader[SvrCnt];
			    	for(int i = 0; i < SvrCnt; i++)
			    		brLogUserNumbers[i] = new BufferedReader(new FileReader(TwitterServer.LogDir 
			    				+ Integer.toString(ConfigManager.ServerPorts.get(i)) + "_user_numbers.log"));
			    	
			    	Long TimePoint = 0L;
			    	int OnLineUsers = 0;
			    	String[] OneLine = null;
			    	ErrorLog UserNumbers = new ErrorLog(TwitterServer.LogDir + "User_numbers.log");
			    	
			    	while(true){
				    	for(int i = 0; i < SvrCnt; i++){
				    		if(OneLine == null){
				    			System.out.println(".");
				    			break;
				    		}
				    		OneLine = brLogUserNumbers[i].readLine().split(":");
				    		TimePoint += Long.valueOf(OneLine[1]);
				    		
				    		OnLineUsers += Long.valueOf(brLogUserNumbers[i].readLine());
				    	}
				    	TimePoint /= SvrCnt;
				    	
				    	UserNumbers.log(new Date(TimePoint).toString());
				    	UserNumbers.log(Integer.toString(OnLineUsers));
				    	
				    	TimePoint = 0L;
				    	OnLineUsers = 0;
			    	}
			    	
			    	UserNumbers.closeLog();
			    	*/
			    	
			    
			    else if(s != null){
			    	System.out.println("Command not supported.");
			    }
			    else
			    	System.exit(0);
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		} 
	}
}