package twitterClient;

import java.io.File;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Date;

import toolBox.ConfigManager;
import toolBox.ErrorLog;

public class TwitterClient {
	
	// TwitterClients' working directory and log directory
	public static String WorkingDir;
	public static String LogDir;
	
	public static long SleepInterval;
	public static long AccelerationFactor;
	
	// The "CheckUpdates" thread checks this boolean to decide whether to stop itself.
	// This boolean is set false by the "UploadStatus" thread.
	static boolean isRunning[]; 
	
	// The error log file of this process
	static ErrorLog ErrLogFile;
	
	// Keep all the threads
	static ArrayList<Thread> ThreadArray;
	
	public static void main(String[] args){
		
		int NumOfThreads = 0; // The number of threads/users within one TwitterClient process
		int NumOfFailedUser = 0; // The number of users who lack of required files thus cannot be started
		String UserID = null;
		String[] TempStrings = null;
		
		// The IP addresses and ports of the server; Selected randomly
		InetAddress ServerAddr = null;
		int ServerPort = 0;
		
		// The error log file to record errors and necessary information
		ErrLogFile = null;
		
		// Record the start up time of TwitterClient
		Date StartTime = null;
		
		try{
			// Create a new error log file
			ErrLogFile = new ErrorLog(args[0]);
			
			// Load the configurations from the configuration file
			if(ConfigManager.LoadClientConfig() == 0){ // Find no server in the configuration file.
				ErrLogFile.logError("No available servers in the confguration file.");
				ErrLogFile.closeLog();
				return;
			}
			
			// Name ourself
			Thread.currentThread().setName("TwitterClientMainThread");
			
			// Build a thread array for checking if all these threads have exited
			ThreadArray = new ArrayList<Thread>();
			isRunning = new boolean[args.length - 1];
			StartTime = new Date();
			
			// Build the threads
		    for (NumOfThreads = 1; NumOfThreads < args.length; NumOfThreads ++) {
		    	
		    		UserID = args[NumOfThreads];
		    		
					// Check whether all required files exist 
					if(	!(new File(WorkingDir + UserID + "/" +  UserID + "_followee").exists()
							&& new File(WorkingDir + UserID + "/" +  UserID + "_event").exists())){
						ErrLogFile.logError(UserID + ": Not all required files exist for this user.");
						NumOfFailedUser++;
						continue;
					}
					
					// Allocate a server for a client
					TempStrings = ConfigManager.SelectRandomServer();
					ServerAddr = InetAddress.getByName(TempStrings[0]); // Get the IP address of a randomly selected server
					ServerPort = Integer.valueOf(TempStrings[1]); // Get the port of the same randomly selected server
					
					// Start the threads
					Thread ThreadUpldStatus = new TweetsSender(ServerAddr, ServerPort + 1, UserID, NumOfThreads - 1);
					ThreadUpldStatus.start();
					
					ThreadArray.add(ThreadUpldStatus);
		    }
		    
		    // Wait for all threads to die
		    for(int i = 0; i < ThreadArray.size(); i++)
		    	ThreadArray.get(i).join();
		   
		}catch(Exception e){
			if(ErrLogFile != null){
				ErrLogFile.logException(e);
				ErrLogFile.logInfo("TwitterClient ends abnormally.");
			}else
				e.printStackTrace();
		}finally{
			if(ErrLogFile != null){
				ErrLogFile.logInfo(Long.toString(NumOfThreads - NumOfFailedUser - 1) + " user(s)"
						+ "\n\tStarts: " + StartTime.toString() 
						+ "\n\tEnds: " + new Date().toString()
						+ "\n\tDuration: " + Long.toString((new Date().getTime() - StartTime.getTime()))
						+ " ms");
				ErrLogFile.closeLog();
			}else
				System.out.println(Long.toString(NumOfThreads - NumOfFailedUser - 1) + " user(s)"
						+ "\n\tStarts: " + StartTime.toString() 
						+ "\n\tEnds: " + new Date().toString()
						+ "\n\tDuration: " + Long.toString((new Date().getTime() - StartTime.getTime()))
						+ " ms");
			
			System.exit(0);
		}
	}
	
	// This method is invoked by the threads we create.
	public static synchronized void Log(String logstr){
		try{
			ErrLogFile.logInfo(logstr);
		}catch(Exception e){
			if(ErrLogFile != null)
				ErrLogFile.logException(e);
			else
				e.printStackTrace();
		}
	}
}

