package twitterServer;

import java.net.ServerSocket;

import toolBox.ConfigManager;
import toolBox.ErrorLog;

public class TwitterServer {
	
	// Server's working directory and log directory
	public static String WorkingDir;
	public static String LogDir;
	
	// Server's port
	public static int ServerPort;
	
	// Server's names
	public static String ServerName;
	public static String ServerLogName;
	// Used by UpdatesChecker: The length of each tweet plus the time it was created should NOT exceed "ReadAheadLimit".
	public static int ReadAheadLimit;
	
	// Used by UpdatesChecker: If we fail to parse a tweet because of its incompleteness, we retry "MaxRetryCount" times before we give up.
	public static int MaxRetryCount;
	
	private static ErrorLog ErrLogFile; // The error log file
	private static long UserThreadCount; // Keep the total number of all "TweetsReceiver" threads. 
	
	public static void main(String[] args){
		
		ServerSocket[] SvrSock;
		
		try {
			// Load server configurations and check whether these configurations are OK.
			if((SvrSock = ConfigManager.LoadServerConfig()) == null){
				System.out.println("Fatal error: Cannot initialize the server.");
				return;
			}
			
			Thread.currentThread().setName("TwitterServerMainThread");
			ServerName = SvrSock[0].getLocalSocketAddress().toString();
			ServerLogName = Integer.toString(SvrSock[0].getLocalPort());
			
			// Create a new log file
			// ErrLogFile = new ErrorLog(TwitterServer.LogDir + "TwitterServer" + ServerPort + ".log");
			
			/* 
			 * Listen to connections of TwitterClient, for periodically checking if there are any 
			 * status updates of one's followees.
			 */
			new Thread(new ListenerForUpdatesChecker(SvrSock[0])).start();
			
			/* 
			 * Listen to connections of TwitterClient, for receiving and serializing new statuses.
			 * This thread can be modified to support more kinds of requests from Cuckoo clients.
			 */
			new Thread(new ListenerForTweetsReceiver(SvrSock[1])).start();
		
			System.out.println("TwitterServer starts: " + ServerName);
			
			/* 
			 * Response to commands from "ServiceController"
			 */
			new Thread(new CommandResponse()).start();
			
			
			// FIX THIS: Check if all threads have ended.
			
		} catch (Exception e) {
			if(ErrLogFile != null){
				ErrLogFile.logException(e);
				ErrLogFile.logInfo("TwitterServer ends abnormally.");
			}else
				e.printStackTrace();
		} finally {
			// FIX THIS
			
		}
	}
	
	static synchronized void AddThread(){
		UserThreadCount++;
	}
	
	static synchronized void RemoveThread(){
		UserThreadCount--;
	}
	
	static synchronized long GetThreadCount(){
		return UserThreadCount;
	}
}
